#ifndef RCPARSER_H__
#define RCPARSER_H__

#include <stdexcept>
#include <string>
#include "Lexer.h"

/*
	09/12/01	Initial creation
*/

/**
	RCParser parses OnDir resource files.
*/

class RCParser : public Lexer {
	public :
		struct exception : public Lexer::exception { 
			exception(string const &str, iterator const &it) : Lexer::exception((string(str) + " (got '" + it.value() + "')").c_str(), it.line()) {} 
			exception(string const &str, unsigned line) : Lexer::exception(str, line) {} 
		};
		RCParser();
		~RCParser();

		void operator () (string const &file);

		string enterPath(string const &path);
		string leavePath(string const &path);

	private :
		// Private, so object can't be default copied.
		RCParser(RCParser const &copy) {}
		RCParser &operator = (RCParser const &copy) { return *this; }

		enum {
			WHITESPACE = 256,
			COMMENT,
			STRING,
			KW_PATH,
			KW_ENTER,
			KW_LEAVE,
		};

		struct Path {
			//Path(string const &name, string const &enter, string const &leave) : name(name), enter(enter), leave(leave) {}
			string name, enter, leave;
		};

		map<string, Path> path;
};

#endif
